package cz.zoubelu.codelist;

import cz.zoubelu.utils.CsvFileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * Created by zoubas.
 * This list represents the application schema of graph integration
 */

public class SystemsList {
    private final Logger log = Logger.getLogger(getClass());

    private static List<SystemApp> systemsList;

    private List<SystemApp> newlyCreated;

    private String fileLocation;

    public SystemsList() {
        this.newlyCreated = new ArrayList<SystemApp>();
        this.systemsList = new ArrayList<SystemApp>() {{
            add(new SystemApp("KUKACKA",100));
            add(new SystemApp("LISKA",101));
            add(new SystemApp("PAVIAN",102));
            add(new SystemApp("HROCH",103));
            add(new SystemApp("FRODO",104));
            add(new SystemApp("POSTOLKA",107));
            add(new SystemApp("BOBR",108));
            add(new SystemApp("EZOP",109));
            add(new SystemApp("ANTILOPA",110));
            add(new SystemApp("FRETKA",111));
            add(new SystemApp("SNAKE",112));
            add(new SystemApp("OREL",113));
            add(new SystemApp("NOREK",114));
            add(new SystemApp("MYS",115));
            add(new SystemApp("KRTEK",116));
            add(new SystemApp("LISAK",141));
            add(new SystemApp("JEZEVEC",142));
            add(new SystemApp("MOUCHA",201));
            add(new SystemApp("TYGR",1000));
            add(new SystemApp("KOLIBRIK",1001));
            add(new SystemApp("UZOVKA",1003));
            add(new SystemApp("HAD",1004));
            add(new SystemApp("MOTYL",1005));
            add(new SystemApp("KOBYLKA",1006));
            add(new SystemApp("PES",1007));
            add(new SystemApp("KOCKA",1008));
            add(new SystemApp("PAPOUSEK",1009));
            add(new SystemApp("KRECEK",1010));
            add(new SystemApp("MORCE",1011));
            add(new SystemApp("HAD",1012));
            add(new SystemApp("PAVOUK",1013));
            add(new SystemApp("KOS",1014));
            add(new SystemApp("KAPR",1015));
            add(new SystemApp("OKOUN",1016));
            add(new SystemApp("STIKA",1017));
            add(new SystemApp("AMUR",1018));
            add(new SystemApp("ZRALOK",1019));
            add(new SystemApp("BELICE",1020));
            add(new SystemApp("KRAB",1021));
            add(new SystemApp("RAK",1022));
            add(new SystemApp("LOSOS",1023));
            add(new SystemApp("MEDVED",1024));
            add(new SystemApp("SRNKA",1025));
            add(new SystemApp("BROUK",0));
            add(new SystemApp("CZGUSRMNG",1));
            add(new SystemApp("CZGUCINFO",2));
            add(new SystemApp("CZGPOLINFO",3));
            add(new SystemApp("CZGPOLICING",4));
            add(new SystemApp("CZGMESSAGING",5));
            add(new SystemApp("CZGCALCBM",6));
            add(new SystemApp("CZGDELIVERY",7));
            add(new SystemApp("CZGPRINTS",8));
            add(new SystemApp("CZGDA",9));
            add(new SystemApp("CZGCLAIMS",12));
            add(new SystemApp("CZGCODELIST",13));
            add(new SystemApp("CZGCARPRICING",14));
            add(new SystemApp("CZGPRODCATALOG",15));
            add(new SystemApp("CZGAGENTINFO",16));
            add(new SystemApp("CZGEARNIX",17));
            add(new SystemApp("CZGSEARCH",18));
            add(new SystemApp("CZGCOMM",19));
            add(new SystemApp("CZGLEADMNG",20));
            add(new SystemApp("CZGBLACKLISTING",21));
            add(new SystemApp("CZGCONTRACTIMPORTPMV",22));
            add(new SystemApp("CZGPROCESSSUPPORT",23));
            add(new SystemApp("CZCTIA",24));
            add(new SystemApp("MONITORING",99));
            add(new SystemApp("MESSAGE HOUSE",134));
            add(new SystemApp("DESET",10));
            add(new SystemApp("NOSOROZEC",555));
            add(new SystemApp("ZIRAFA",1026));
            add(new SystemApp("ZEBRA",1055));
            add(new SystemApp("KURE",1056));
            add(new SystemApp("123",123));
            add(new SystemApp("IMOTEP",5097));
        }};
    }

    /**
     * Methods for resolving system
     */

    public SystemApp getSystemByID(Integer id) {
        for (SystemApp system : systemsList) {
            if ((id).equals(system.getId())) {
                return system;
            }
        }
        log.info("Unable to find system with id: " + id + ".");
        return addSystem(id.toString(), id);
    }

    public SystemApp getSystemByName(String nameLCase) {
        String name = StringUtils.upperCase(nameLCase);
        for (SystemApp system : systemsList) {
            if (name.equals(system.getName())) {
                return system;
            }
        }
        log.info("Unable to find system with name: " + name + ".");
        return addSystem(name, generateUniqueSystemId());
    }

    private SystemApp addSystem(String name, Integer id) {
        log.info(String.format("Adding new system to the system list. Name: %s id: %s.", name, id));
        SystemApp s = new SystemApp(name, id);
        systemsList.add(s);
        newlyCreated.add(s);
        return s;
    }

    private Integer generateUniqueSystemId() {
        log.error("Generating new Id for system. Please check the newly generated apps and fix values.");

        Integer rNmbr = new Random().nextInt(5000 - 3001) + 3101;

        for (SystemApp system : systemsList) {
            if (system.getId().equals(rNmbr)) {
                generateUniqueSystemId();
            }
        }
        return rNmbr;
    }

    /**
     * Methods working with list file.
     */
    public List<SystemApp> loadList() {
        this.systemsList = CsvFileUtils.load(fileLocation);
        return values();
    }

    public void saveList() {

        if (!values().isEmpty() && values() != null) {
            CsvFileUtils.save(values(), fileLocation);
        } else {
            log.error("Failed to save system list because it's empty.");
        }
    }

    public boolean listFileExists() {
        return CsvFileUtils.checkFile(fileLocation);
    }

    /**
     * LIST CRUD ops
     */
    public void remove(SystemApp system) {
        SystemApp systemApp = getSystemByID(system.getId());
        if (systemApp != null) {
            systemsList.remove(systemApp);
        } else {
            throw new IllegalArgumentException("Cannot remove from list, can't find the system with id: " + system.getId());
        }
    }

    public void edit(SystemApp system) {
        remove(system);
        systemsList.add(system);
    }

    public void add(SystemApp system) {
        systemsList.add(system);
    }


    /** GETTERS **/
    public void setFileLocation(String fileLocation) {
        this.fileLocation = fileLocation;
    }

    public List<SystemApp> getNewlyCreated() {
        return newlyCreated;
    }

    public static List<SystemApp> values() {
        return systemsList;
    }
}
