package cz.zoubelu.controller;

import cz.zoubelu.codelist.SystemApp;
import cz.zoubelu.codelist.SystemsList;
import cz.zoubelu.domain.Application;
import cz.zoubelu.domain.ConsumeRelationship;
import cz.zoubelu.domain.Method;
import cz.zoubelu.model.ConversionError;
import cz.zoubelu.model.TimeRange;
import cz.zoubelu.repository.ApplicationRepository;
import cz.zoubelu.service.DataConversion;
import cz.zoubelu.service.Visualization;
import cz.zoubelu.utils.ErrorResponse;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.sql.Timestamp;
import java.util.List;
import java.util.Map;

/**
 * Created by zoubas.
 */

@RestController
@RequestMapping(value = "/api")
public class RestGraphController {
    private final Logger log = Logger.getLogger(getClass());

    @Autowired
    private ApplicationRepository applicationRepo;

    @Autowired
    private Visualization visualization;

    @Autowired
    private DataConversion dataConversion;

    /**
     * @param systemId
     * @return Application with specified system ID
     */
    @RequestMapping(value = "/application/{systemId}", method = RequestMethod.GET)
    public Application getApplication(@PathVariable("systemId") Integer systemId) {
        return applicationRepo.findBySystemId(systemId);
    }


    /**
     * @param systemId
     * @return Methods that are being consumed by application with specified system ID.
     */
    @RequestMapping(value = "/application/{systemId}/consumes", method = RequestMethod.GET)
    public List<ConsumeRelationship> getConsumedMethods(@PathVariable("systemId") Integer systemId) {
        Application app = applicationRepo.findBySystemId(systemId);
        return app.getConsumeRelationship();
    }

    @RequestMapping(value = "/application/{appName}/info", method = RequestMethod.GET)
    public Application getApplicationInfo(@PathVariable("appName") String appName) {
        return applicationRepo.findByName(StringUtils.upperCase(appName));
    }

    /**
     * @param systemId
     * @return Method being provided by the application with specified system ID.
     */
    @RequestMapping(value = "/application/{systemId}/provides", method = RequestMethod.GET)
    public List<Method> getProvidedMethods(@PathVariable("systemId") Integer systemId) {
        cz.zoubelu.domain.Application app = applicationRepo.findBySystemId(systemId);
        return app.getProvidedMethods();
    }

    /**
     * Creating JSON which is source for graph view (visualisation.html)
     *
     * @return Map<String,Object> of all the nodes and relations in the graph as JSON, one map row is one relation ('Node - Relation - Node') record.
     */
    @RequestMapping("/graph")
    public Map<String, Object> visualiseWholeGraph() {
       return visualization.visualizeGraph();
    }

    @RequestMapping(value = "/codelist", method = RequestMethod.GET)
    public List<SystemApp> getSystemsList() {
        return SystemsList.values();
    }


    /* THIS METHOD IS ONLY FOR LOCAL TESTING USAGE */
    @RequestMapping(value = ("/insert"), method = RequestMethod.GET)
    public Object insertData(@RequestParam String year,@RequestParam String month,@RequestParam String day, @RequestParam String hour) {
        Timestamp start = Timestamp.valueOf(year+"-"+month+"-" + day + " 00:00:00.0");
        Timestamp end = Timestamp.valueOf(year+"-"+month+"-" +day +" " + hour + ":00:00.0");
        long f = System.currentTimeMillis();
        List<ConversionError> errors = dataConversion.convertData("MESSAGE", new TimeRange(start, end));
        long l = System.currentTimeMillis();
        log.info("--------------------------------------------------------------------------------");
        log.info("Conversion took: " + (l-f));
        log.info("--------------------------------------------------------------------------------");
        return errors.size()>0 ?errors: "Done in: " + (l-f) + " ms.";
    }


    @ExceptionHandler(Exception.class)
    @ResponseBody
    public ErrorResponse returnError(Exception e) {
        return new ErrorResponse(e.getMessage());
    }
}
