package cz.zoubelu.repository.impl;

import cz.zoubelu.model.TimeRange;
import cz.zoubelu.repository.MessageRepository;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowCallbackHandler;

import java.util.List;

/**
 * Created by zoubas.
 */
public class MessageRepositoryImpl implements MessageRepository {

    private JdbcTemplate jdbcTemplate;

    private static final String QUERY_BY_TIMERANGE = "select ID,REQUEST_TIME,RESPONSE_TIME,APPLICATION,MSG_TYPE,MSG_VERSION,MSG_SRC_SYS,MSG_TAR_SYS from $tablename where request_time between ? and ?";

    private static final String QUERY_WHOLE_TABLE = "select ID,REQUEST_TIME,RESPONSE_TIME,APPLICATION,MSG_TYPE,MSG_VERSION,MSG_SRC_SYS,MSG_TAR_SYS from $tablename";

    private static final String RETRIEVE_TABLE_NAME = "select table_name from all_tables where table_name like 'A_MESSAGE%' order by table_name";

    public MessageRepositoryImpl(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
        this.jdbcTemplate.setFetchSize(40000);
    }

    public void fetchAndConvertData(String tableName, RowCallbackHandler handler) {
        String query = QUERY_WHOLE_TABLE.replace("$tablename", tableName);
        this.jdbcTemplate.query(query, handler);
    }

    public void fetchAndConvertData(String tableName, final TimeRange timeRange, RowCallbackHandler handler) {
        if (timeRange != null) {
            String query = QUERY_BY_TIMERANGE.replace("$tablename", tableName);
            this.jdbcTemplate.query(query, handler, timeRange.getStartDate(),
                    timeRange.getEndDate());
        } else {
            fetchAndConvertData(tableName, handler);
        }
    }

    public List<String> getDatabaseMessagesTables() {
        return jdbcTemplate.queryForList(RETRIEVE_TABLE_NAME, String.class);
    }

    @Override
    public boolean isTableInDatabase(String tableName) {
        List<String> tableNames = getDatabaseMessagesTables();
        return tableNames.contains(tableName);
    }
}
