package cz.zoubelu.task;

import cz.zoubelu.model.ConversionError;
import cz.zoubelu.model.Frequency;
import cz.zoubelu.model.TimeRange;
import cz.zoubelu.service.DataConversion;
import cz.zoubelu.service.impl.EmailSender;
import cz.zoubelu.utils.DateUtils;
import it.sauronsoftware.cron4j.SchedulingPattern;
import it.sauronsoftware.cron4j.Task;
import it.sauronsoftware.cron4j.TaskExecutionContext;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

/**
 * Created by zoubas.
 */
public class ConversionTask extends Task {

    @Autowired
    private DataConversion dataConversion;

    private final Logger log = Logger.getLogger(getClass());

    private String scheduledTaskId;

    private SchedulingPattern pattern;

    @Autowired
    private EmailSender emailSender;

    private static final String TABLE_NAME = "MESSAGE";

    @Override
    public void execute(TaskExecutionContext taskExecutionContext) throws RuntimeException {
        log.info("Starting conversion.");
        long startTime = System.currentTimeMillis();

        Frequency frequency = DateUtils.recogniseSchedulerFrequency(pattern.toString());

        TimeRange timeRange = DateUtils.getTimeRangeByFrequency(frequency, pattern.toString());

        String tableName = getTableName(frequency);
        log.info("Recognised frequency is " + frequency + ". Querying table: " + tableName);

        List<ConversionError> errors = dataConversion.convertData(tableName, timeRange);

        long stopTime = System.currentTimeMillis();
        log.info(String.format("Conversion took %s ms, -> %s minutes.", stopTime - startTime,
                ((stopTime - startTime) / 60000)));
        log.info("Conversion ended with " + errors.size() + " errors.");
        sendEmail(errors);

        log.info("Clearing cache.");
        dataConversion.clearCache();
    }

    private void sendEmail(List<ConversionError> errors) {
        if (errors.size() > 0 && !errors.isEmpty()) {
            emailSender.sendEmail(errors);
        }
    }

    private String getTableName(Frequency frequency) {
        if (frequency.equals(Frequency.DAILY)) return TABLE_NAME;
        if (frequency.equals(Frequency.WEEKLY)) return TABLE_NAME;
        if (frequency.equals(Frequency.MONTHLY)) return "A_" + TABLE_NAME + "_" + DateUtils.getYearMonthSuffix();
        throw new IllegalArgumentException("Cannot get table name for frequency " + frequency);
    }

    public String getScheduledTaskId() {
        return scheduledTaskId;
    }

    public void setScheduledTaskId(String scheduledTaskId) {
        this.scheduledTaskId = scheduledTaskId;
    }

    public SchedulingPattern getPattern() {
        return pattern;
    }

    public void setPattern(SchedulingPattern pattern) {
        this.pattern = pattern;
    }
}
