function showLinearGraph(circle) {
    var linearGraph = document.getElementById("linearGraph");
    wipe(linearGraph)
    // Set the dimensions of the canvas / graph
    var margin = {top: 30, right: 20, bottom: 70, left: 100},
        width = 600 - margin.left - margin.right,
        height = 400 - margin.top - margin.bottom;

    // Parse the day / time
    var parseDate = d3.time.format("%d-%m-%Y").parse;

    // Set the ranges
    var x = d3.time.scale().range([0, width]);
    var y = d3.scale.linear().range([height, 0]);

    // Define the axes
    var xAxis = d3.svg.axis().scale(x)
        .orient("bottom").ticks(5);

    var yAxis = d3.svg.axis().scale(y)
        .orient("left").ticks(5);

    // Define the line
    var usageline = d3.svg.line()
        .x(function (d) {
            return x(d.date);
        })
        .y(function (d) {
            return y(d.count);
        });



    // Adds the svg canvas
    var svg = d3.select("#linearGraph")
        .append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom)
        .append("g")
        .attr("transform",
            "translate(" + margin.left + "," + margin.top + ")");

    // Get the data
    var URL = document.URL;

    d3.json(URL + "api/" + circle.title + "/dailyUsage", function (error, data) {
        data.forEach(function (d) {
            d.date = parseDate(d.day);
            d.count = +d.count;
        });

        // Scale the range of the data
        x.domain(d3.extent(data, function (d) { return d.date; }));
        y.domain([0, d3.max(data, function (d) { return d.count; })]);

        // Nest the entries by symbol
        var dataNest = d3.nest()
            .key(function (d) {
                return d.method;
            })
            .entries(data);
        var color = d3.scale.category10();   // set the colour scale

        legendSpace = width / methodCount(data); // spacing for legend

        // Loop through each symbol / key
        dataNest.forEach(function (d, i) {

            svg.append("path")
                .attr("class", "line")
                .style("stroke", function () { // Add the colours dynamically
                    return d.color = color(d.key);
                })
                .attr("d", usageline(d.values));

            // Add the Legend
            svg.append("text")
                .attr("x", (legendSpace / 2) + i * legendSpace) // spacing
                .attr("y", height + (margin.bottom / 2) + 5)
                .attr("class", "legend")    // style the legend
                .style("fill", function () { // dynamic colours
                    return d.color = color(d.key);
                })
                .text(d.key);

        });

        // Add the X Axis
        svg.append("g")
            .attr("class", "x axis")
            .attr("transform", "translate(0," + height + ")")
            .call(xAxis);

        // Add the Y Axis
        svg.append("g")
            .attr("class", "y axis")
            .call(yAxis);

    })
};

function methodCount(data) {
    var lookup = {};
    var result = [];

    for (var method, i = 0; method = data[i++];) {

        var name = method.method;
        if (!(name in lookup)) {
            lookup[name] = 1;
            result.push(name);
        }
    }
    return result.length;
}