package cz.zoubelu.integration;

import cz.zoubelu.codelist.SystemApp;
import cz.zoubelu.model.Frequency;
import cz.zoubelu.service.impl.EmailSender;
import cz.zoubelu.model.ConversionError;
import cz.zoubelu.utils.CsvFileUtils;
import cz.zoubelu.utils.DateUtils;
import cz.zoubelu.model.TimeRange;
import it.sauronsoftware.cron4j.SchedulingPattern;
import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 * Created by zoubas.
 */
public class UtilTest extends AbstractTest {

    public static final String SYSTEMS_LIST_TEST_CSV = "SYSTEMS_LIST_test.csv";

    @Test
    public void testTimeRangeCreation() {
        TimeRange range = DateUtils.getTimeRange();
        Assert.assertEquals("00:00:00.0", range.getStartDate().toString().substring(11, 21));
        Assert.assertEquals("23:59:59.9", range.getEndDate().toString().substring(11, 21));
    }

    @Test
    public void testTableSuffixCreation() {
        String tableName = DateUtils.getYearMonthSuffix();
        Assert.assertTrue(tableName.length() == 6);
    }

    @Test
    public void testSystemListSave() {
        List<SystemApp> systemApps = new ArrayList<SystemApp>();
        systemApps.add(new SystemApp("ONE", 1));
        systemApps.add(new SystemApp("TWO", 2));
        systemApps.add(new SystemApp("THREE", 3));
        CsvFileUtils.save(systemApps, SYSTEMS_LIST_TEST_CSV);
    }

    @Test
    public void testSystemListLoad() {
        testSystemListSave();
        List<SystemApp> systemApps = CsvFileUtils.load(SYSTEMS_LIST_TEST_CSV);
        Assert.assertEquals(systemApps.size(), 3);
    }

    @Test
    public void shouldRecogniseTypeOfSchedule() {
        SchedulingPattern dailyPattern = new SchedulingPattern("59 11 * * *");
        SchedulingPattern weeklyPattern = new SchedulingPattern("45 8 * * Sun");
        SchedulingPattern monthlyPattern = new SchedulingPattern("00 00 7 * *");

        Frequency daily = DateUtils.recogniseSchedulerFrequency(dailyPattern.toString());
        Frequency weekly = DateUtils.recogniseSchedulerFrequency(weeklyPattern.toString());
        Frequency monthly = DateUtils.recogniseSchedulerFrequency(monthlyPattern.toString());

        Assert.assertEquals(Frequency.DAILY, daily);
        Assert.assertEquals(Frequency.WEEKLY, weekly);
        Assert.assertEquals(Frequency.MONTHLY, monthly);
    }

    @Test
    @Ignore("sends email if not ignored.")
    public void sendEmail() {
        EmailSender emailSender = new EmailSender();
        List<ConversionError> errors = new ArrayList<ConversionError>();
        errors.add(new ConversionError("Somethings wrong"));
        errors.add(new ConversionError("This should never happen."));
        emailSender.sendEmail(errors);
    }

    @Test
    public void shouldReturnCorrectTimeRangeForTable() {
        List<TimeRange> timeRanges = DateUtils.getDaysTimeRangesOfMonth("A_MESSAGE_201601");
        Assert.assertEquals(31, timeRanges.size());
        Calendar cal = Calendar.getInstance();

        for (TimeRange dayRange : timeRanges) {
            cal.setTime(dayRange.getStartDate());
            Assert.assertEquals(2016, cal.get(Calendar.YEAR));
            Assert.assertEquals(1, cal.get(Calendar.MONTH)+1);
        }
    }
}
